/*	$NetBSD: setjmp.S,v 1.5 2003/04/05 23:08:51 bjh21 Exp $	*/

/*
 * Copyright (c) 1997 Mark Brinicombe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Mark Brinicombe
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <machine/asm.h>
#include <machine/setjmp.h>

#define SAVED_R4 0
#define SAVED_R5 4
#define SAVED_R6 8
#define SAVED_PC 12
#define SAVED_SP 16
#define SAVED_SIGMASK 20
/*
 * C library -- setjmp, longjmp
 *
 *	longjmp(a,v)
 * will generate a "return(v)" from the last call to
 *	setjmp(a)
 * by restoring registers from the stack.
 * The previous signal state is restored.
 */

proc _setjmp
	mov er4, @(SAVED_R4, er0)          // save registers
	mov er5, @(SAVED_R5, er0)
	mov er6, @(SAVED_R6, er0)
	mov sp, @(SAVED_SP, er0)

	mov @sp, er1                       // return address
	mov er1, @(SAVED_PC, er0)

	mov er0, er4                       // er4 now contains jmpbuf pointer

	sub er0, er0
	jsr _sigblock                      // block signals
	mov er0, @(SAVED_SIGMASK, er4)     // save old signal mask

	mov @(SAVED_R4, er4), er4          // restore er4, which we used

	sub er0, er0                       // return
	rts

proc _longjmp
	mov er0, er4                       // er4 now contains jmpbuf pointer
	mov er1, er5                       // er5 now contains return value

	mov @(SAVED_SIGMASK, er4), er0
	jsr _sigsetmask                    // restore signal mask

	mov er4, er1                       // er1 now contains jmpbuf pointer
	mov er5, er0                       // er0 now contains return value

	mov @(SAVED_R4, er1), er4          // restore registers
	mov @(SAVED_R5, er1), er5
	mov @(SAVED_R6, er1), er6
	mov @(SAVED_SP, er1), sp

	mov @(SAVED_PC, er1), er1          // return address
	add #4, sp                         // in lieu of rts
	jmp @er1
